"""Playyaml file."""

import os
import sys
from pprint import pprint

import yaml

from . import config


def check_console():
    """Check if CONFIG.console is not none."""
    if config.CONSOLE is None:
        from rich.console import Console

        config.CONSOLE = Console()


def load_yaml(filename):
    """Load yaml file from disk.

    Args:
        filename: file to load

    Returns:
        struct from yaml file
    """
    try:
        with open(os.path.expanduser(filename), "r") as file:
            data = yaml.safe_load(file)
    except Exception as e:  # noqa:B902
        sys.stderr.write(f"can't open config file: {filename}  {e}")
        exit(1)
    return data


def play_command_loadxml(task):
    """Play command loadxml.

    Args:
        task: task array

    Returns:
        Good: boolean
    """
    from .xmltv_load_save import xmltv_load

    check_console()
    store = task.get("store", "default")
    if "file" not in task:
        sys.stderr.write(f"missing file entry in task {str(task)}")
        exit(3)
    file = task.get("file")
    with config.CONSOLE.status(f"Loading...{file} to store {store}", spinner="dots"):
        config.STORE[store] = xmltv_load(file)
    print(f"Loaded {file} to store {store}")
    return True


def play_command_savexml(task):
    """Play command savexml.

    Args:
        task: task array

    Returns:
        Good: boolean
    """
    from .xmltv_load_save import xmltv_save

    check_console()
    store = task.get("store", "default")
    if "file" not in task:
        sys.stderr.write(f"missing file entry in task {str(task)}")
        exit(3)
    file = task.get("file")
    with config.CONSOLE.status(f"Loading...{file} to store {store}", spinner="dots"):
        xmltv_save(file, config.STORE[store])
    print(f"Saved {file} to store {store}")
    return True


def play_command_only_channels(task):
    """Play command only_channels.

    Args:
        task: task array

    Returns:
        Good: boolean
    """
    from .xmltv_channels import channels_remove_all_channels_not_in_list
    from .xmltv_programs import programme_remove_all_channels_not_in_list

    check_console()
    store = task.get("store", "default")
    if "channels" not in task:
        sys.stderr.write(f"missing channels entry in task {str(task)}")
        exit(3)
    channels = task.get("channels")
    with config.CONSOLE.status(f"Removing channels from store {store}", spinner="dots"):
        config.STORE[store] = channels_remove_all_channels_not_in_list(
            config.STORE[store], channels
        )
    with config.CONSOLE.status(
        f"Removing programme from store {store}", spinner="dots"
    ):
        config.STORE[store] = programme_remove_all_channels_not_in_list(
            config.STORE[store], channels
        )
    print(f"Removed channels and programs from store {store}")
    return True


def play_command_add(task):
    """Play command add 2 xmltv stores.

    Args:
        task: task array

    Returns:
        Good: boolean
    """
    from .xmltv_channels import join_channels
    from .xmltv_programs import join_programs

    check_console()
    store = task.get("store", "default")
    if "add_store" not in task:
        sys.stderr.write(f"missing add_store entry in task {str(task)}")
        exit(3)
    add_store = task.get("add_store")

    with config.CONSOLE.status("Join channels...", spinner="dots"):
        config.STORE[store] = join_channels(
            config.STORE[store], config.STORE[add_store]
        )

    with config.CONSOLE.status("Join programs...", spinner="dots"):
        config.STORE[store] = join_programs(
            config.STORE[store], config.STORE[add_store]
        )

    print(f"Add channels and programs from store {add_store} to store {store}")
    return True


def play_command_change_timezone(task):
    """Change timezone in program data.

    Args:
        task: task array

    Returns:
        Good: boolean
    """
    from .xmltv_programs import search_and_replace_in_start_and_stop

    check_console()
    store = task.get("store", "default")
    if "search" not in task:
        sys.stderr.write(f"missing search entry (search string) in task {str(task)}")
        exit(3)
    if "replace" not in task:
        sys.stderr.write(f"missing replace entry (replace string) in task {str(task)}")
        exit(3)
    search = task.get("search")
    replace = task.get("replace")

    with config.CONSOLE.status(
        "Search and replace in start/stop program time...", spinner="dots"
    ):
        config.STORE[store] = search_and_replace_in_start_and_stop(
            config.STORE[store], search, replace
        )

    print(
        f"Search and replace in start/stop time programs: store {store} search:{search} / replace:{replace}"
    )
    return True


def play_command_analyse_programs(task):
    """Analyse programs for a store.

    Args:
        task: task array

    Returns:
        Good: boolean
    """
    from .xmltv_programs import display_table_analyse_programs

    check_console()
    store = task.get("store", "default")
    title = task.get("title", None)
    display_table_analyse_programs(
        config.CONSOLE, config.STORE[store], add_source_column=True, source_name=title
    )
    return True


def play_command_execute_command(task):
    """Execute a program.

    Args:
        task: task array

    Returns:
        Good: boolean
    """
    from .playyaml_funcs import execute_command

    if "execute" not in task and "execute_base64" not in task:
        sys.stderr.write(f"missing execute or execute_base64 entry in task {str(task)}")
        exit(3)

    if "execute_base64" in task:
        from .playyaml_funcs import base64_2_str

        execute = base64_2_str(task.get("execute_base64"))
    else:
        execute = task.get("execute")
    shell = task.get("shell", False)
    capture_output = task.get("capture_output", True)

    result = execute_command(
        execute, shell=shell, capture_output=capture_output  # noqa:S604
    )  # noqa:S604
    if capture_output:
        print(result.stdout)
        print(result.stderr)
        print(f"error_code: {result.returncode}")
    return True


def task_check_tag(task, include_tags=None, exclude_tags=None):
    """Check if the task have a tag and if we skip or not this task.

    Args:
        task: task array
        include_tags: list of tags to run
        exclude_tags: list of tags to skip

    Returns:
        RunTask: boolean, True run task, False skip task
    """
    from fhs_xmltv_tools.playyaml_funcs import check_items_in_2_lists

    temp_tags = task.get('tags', [])
    tags = temp_tags if isinstance(temp_tags, list) else [temp_tags]

    if include_tags is not None and include_tags != []:  # noqa: SIM102
        # check is include tags in in tags of this task, if false then skip this task
        if check_items_in_2_lists(include_tags, tags) is False:
            return False

    if exclude_tags is not None and exclude_tags != []:  # noqa: SIM102
        # check is exclude tags in in tags of this task, if true then skip this task
        if check_items_in_2_lists(exclude_tags, tags) is True:
            return False

    return True



def play_task(task, include_tags=None, exclude_tags=None):
    """Play a task.

    Args:
        task: task array
        include_tags: list of tags to run
        exclude_tags: list of tags to skip

    Returns:
        Good: boolean
    """
    if task_check_tag(task, include_tags=include_tags, exclude_tags=exclude_tags):
        print(f"running task: {task.get('name', 'unknown')}")
    else:
        print(f"skipping task: {task.get('name', 'unknown')}")
        return True

    if "command" not in task:
        sys.stderr.write(f"missing command entry in task {str(task)}")
        exit(3)

    if task["command"] == "analyse_programs":
        return play_command_analyse_programs(task)

    if task["command"] == "loadxml":
        return play_command_loadxml(task)

    if task["command"] == "only_channels":
        return play_command_only_channels(task)

    if task["command"] == "savexml":
        return play_command_savexml(task)

    if task["command"] == "add":
        return play_command_add(task)

    if task["command"] == "change_timezone":
        return play_command_change_timezone(task)

    if task["command"] == "execute_command":
        return play_command_execute_command(task)

    pprint(task)
    return True


def play(commandfile, include_tags=None, exclude_tags=None):
    """Play commandfile.

    Args:
        commandfile: yaml file with instructions
        include_tags: list of tags to run
        exclude_tags: list of tags to skip

    Returns:
        None
    """
    data = load_yaml(commandfile)

    if "tasks" not in data:
        sys.stderr.write("missing tasks entrie in yaml file")
        exit(2)

    for task in data["tasks"]:
        play_task(task, include_tags=include_tags, exclude_tags=exclude_tags)

    return None



"""Extra functions for command playyaml."""

import subprocess  # noqa:S404


def execute_command(command, shell=False, capture_output=True, text=True):
    """Execute a command..

    Args:
        command: command to execute as array (shell=False) or as string (shell=True)
        shell: boolean using shell (like pipe commands in bash)
        capture_output: capture output from command
        text: output is Text

    Returns:
        output of subprocess  .stdout / .stderr / .returncode
    """
    return subprocess.run(  # noqa:DUO116
        command, shell=shell, capture_output=capture_output, text=text  # noqa:S602
    )


def base64_2_str(base64_message):
    """Convert base64 message to text.

    Args:
        base64_message: base64 message you want to decode

    Returns:
        message: as type str
    """
    import base64

    base64_bytes = base64_message.encode("ascii")
    message_bytes = base64.b64decode(base64_bytes)
    message = message_bytes.decode("ascii")
    return message


def check_items_in_2_lists(list1, list2):
    """Check items in 2 lists returns True in at least one item matches.

    Args:
        list1: list of strings
        list2: list of strings

    Returns:
        bool: minimal one item matched
    """
    for i in list1:
        if i in list2:
            return True
    return False




"""xmltv_programs script for fhs_xmltv_tools."""

from . import config  # noqa: F401
from .func_xmltv import get_program_desc, get_program_title


def xmltv_programs(xmltv_data, channel_id):
    """List programs for channel id.

    Args:
        xmltv_data: data object with xmltv data.
        channel_id: channel id we want to search programs for.

    Returns:
        list of programs in xmltv_data.programme format
    """
    list_programs = []
    for p in xmltv_data.programme:
        if p.channel == channel_id:
            list_programs.append(p)
    return list_programs


def xmltv_programs_output(program):
    """Ouput a string with a program name string.

    Args:
        program: item of xmltv_data.programme

    Returns:
        string with title start time and stop time of program.
    """
    return f"{get_program_title(program)} {program.start} - {program.stop}"


def xmltv_programs_test(xmltv_data, channel_id):
    """Create string list of al programs on channel.

    Args:
        xmltv_data: data object with xmltv data.
        channel_id: channel id we want to search programs for.

    Returns:
        list of string: output of xmltv_programs_output
    """
    programs = []
    for p in xmltv_programs(xmltv_data, channel_id):
        programs.append(xmltv_programs_output(p))
    return programs


def analyse_program_initialize(program_data):
    """Return analyse program item.

    Args:
        program_data: item of xmltv_data.programma

    Returns:
        dicts with programs, first start and last_stop fields
    """
    ad = {
        "programs": 0,
        "first_start": program_data.start,
        "last_stop": program_data.stop,
    }
    return ad


def analyse_program_update(current_data, program_data):
    """Update a analyse program item.

    Args:
        current_data: current analayze program item
        program_data: xmltv_data.programma item

    Returns:
        changes current_data
    """
    current_data["programs"] += 1
    if program_data.start < current_data["first_start"]:
        current_data["first_start"] = program_data.start
    if program_data.stop > current_data["last_stop"]:
        current_data["last_stop"] = program_data.stop
    return current_data


def analyse_programs(xmltv_data, filter=None, filter_argument=None):
    """Analyze programs from xmltv_data.

    Gives information about per channel howmany programs and what the first
    start time is and the last stop time.

    Args:
        xmltv_data: data object with xmltv data.
        filter: filter function
        filter_argument: extra arguments for the filter function

    Returns:
        data dicts with key channel id and field value dicts from
        analyse_program_initialize
    """
    data = {}

    if filter is not None:
        programme = filter(xmltv_data, filter_argument)
    else:
        programme = xmltv_data.programme

    for p in programme:
        current_data = data.get(p.channel, None)
        if current_data is None:
            current_data = analyse_program_initialize(p)
        current_data = analyse_program_update(current_data, p)
        data[p.channel] = current_data
    return data


def search_program_create_program_return(program_data):
    """Return search_program item.

    Args:
        program_data: item of xmltv_data.programma

    Returns:
        dicts with channel, title, discription, start, stop
    """
    ad = {
        "channel": program_data.channel,
        "title": get_program_title(program_data),
        "description": get_program_desc(program_data),
        "start": program_data.start,
        "stop": program_data.stop,
    }
    return ad


def search_programs(xmltv_data, search_str, force_case=False):
    """Search all the programs and return list of results.

    Args:
        xmltv_data: data object with xmltv data.
        search_str: string object or regex search.
        force_case: case sensitive search or not

    Returns:
        list of search_Program_create_program_return items
    """
    result = []
    import re

    if force_case:
        search_re = re.compile(search_str)
    else:
        search_re = re.compile(search_str, re.IGNORECASE)

    programme = xmltv_data.programme
    for p in programme:
        if not search_re.search(get_program_title(p)):
            continue
        result.append(search_program_create_program_return(p))
    return result


def programme_remove_all_channels_not_in_list(xmltv_data, channel_set):
    """Remove all programs on channels that are not in the list.

    Args:
        xmltv_data: data object with xmltv data.
        channel_set: set of channel id you want to keep

    Returns:
        xmltv_data
    """
    for p in range(len(xmltv_data.programme), 0, -1):
        if (xmltv_data.programme[p - 1]).channel not in channel_set:
            del xmltv_data.programme[p - 1]
    return xmltv_data


def join_programs(xmltv_data, xmltv_data_add):
    """Add the programs from xmltv_data_add to the channels from xmtv_data.

    Args:
        xmltv_data: data object with xmltv data.
        xmltv_data_add: data object with xmltv data jou want to add.

    Returns:
        xmltv_data
    """
    for p in xmltv_data_add.programme:
        xmltv_data.programme.append(p)
    return xmltv_data


def search_and_replace_in_start_and_stop(xmltv_data, search, replace):
    """Replace in all programs in the start and stop fields the search with replace.

    Use this for example to change the timezone van ' +0000' to ' +0200'

    Args:
        xmltv_data: data object with xmltv data.
        search: string to search in start and stop
        replace: string to replace the 'search' string with in start and stop

    Returns:
        xmltv_data
    """
    for p in xmltv_data.programme:
        p.start = p.start.replace(search, replace)
        p.stop = p.stop.replace(search, replace)
    return xmltv_data


def display_table_analyse_programs(
    console, xmltv_data, add_source_column=False, source_name=None
):
    """Display table analyse programs.

    Args:
        xmltv_data: data object with xmltv data.
        console: rich.console object
        add_source_column: add a source info column
        source_name: name to use for source_info_name
    """
    from rich.console import Console
    from rich.table import Table

    if console is None:
        console = Console()

    with console.status("Analysing...", spinner="dots"):
        result = analyse_programs(xmltv_data)

    if source_name is None:
        source_name = xmltv_data.source_info_name

    table = Table(title=f"Channels: {source_name}")
    if add_source_column:
        table.add_column("Source name", style="green")
    table.add_column("Id", style="cyan")
    table.add_column("start time", style="green")
    table.add_column("end time", style="cyan")
    table.add_column("programs", justify="right", style="green")
    for p in result:
        if add_source_column:
            table.add_row(
                source_name,
                p,
                result[p]["first_start"],
                result[p]["last_stop"],
                str(result[p]["programs"]),
            )
        else:
            table.add_row(
                p,
                result[p]["first_start"],
                result[p]["last_stop"],
                str(result[p]["programs"]),
            )
    console.print(table)
